/*
 *  Area of circle      A = pi*r^2
 *  Area of rectangle   B = (2*r)^2
 *                    A/B = pi/4
 *  We can get A/B numerically by throwing darts at a rectangle of size r
 *  and checking if the coordinates of impact (x,y) lie inside the circle,
 *  which is the case iff sqrt(x^2+y^2) < r.
 *
 *  Of course there are much better ways to approximate pi. However,
 *  this method is used as a quality test for random number generators!
 */

#include <mpi.h>
#include <math.h>
#include <time.h>
#include <stdio.h>
#include <stdlib.h>

int main(int argc, char** argv) {

	MPI_Init(NULL, NULL);

	// get MPI rank (process id) and communicator size

	int rank, size;
	MPI_Comm_rank(MPI_COMM_WORLD, &rank);
	MPI_Comm_size(MPI_COMM_WORLD, &size);

	if (argc < 2) {
		fprintf(stderr, "Usage: %s [num-throws]\n", argv[0]);
		MPI_Abort(MPI_COMM_WORLD, -1);
	}

	// initialize random number generator using the current time
	srand(time(NULL));

	unsigned long int throws = 0;
	unsigned long int tot_throws = atol(argv[1]);
	unsigned long int in_circle = 0;
	unsigned long int max_throws =  tot_throws / size;

	while (throws < max_throws) {

		// do a random throw on a rectangle which has a side length of 1
		// the random variables have to be uniformly distributed

		double x = rand() / ((double) RAND_MAX);
		double y = rand() / ((double) RAND_MAX);

		// check if it inside the circle

		if (sqrt(x*x+y*y) < 1.0) in_circle++;
		throws++;

	}

	unsigned long int sum;
	MPI_Reduce(&in_circle, &sum, 1, MPI_UNSIGNED_LONG, MPI_SUM, 0, MPI_COMM_WORLD);

	double pi = 4.0*(((double)sum / (double)tot_throws));

	if (rank == 0) {
		printf("Approximation of pi after %lu throws is %lf\n", tot_throws, pi);
	}

	MPI_Finalize();

}

